@@ -1,17 +1,19 @@
+import re
 import types
 from abc import ABC
 from inspect import isfunction, isclass
+
 from numpy import isnan
-from pandas import DataFrame
+from pandas import DataFrame, Series
 from tqdm.auto import tqdm
-import re
+
+from ._type_converter import type2str, is_equal, txt_to_lines
 from .section_labels import *
-from ._type_converter import type2str, is_equal
 from .section_lists import LINK_SECTIONS, NODE_SECTIONS
 
 SWMM_VERSION = '5.1.015'
 
-inp_sep = ';;' + "_" * 100
+SEP_INP = ';;' + "_" * 100
 
 
 ########################################################################################################################
@@ -83,20 +85,6 @@ class CustomDict:
 
 
 class CustomDictWithAttributes(CustomDict):
-    def __setitem__(self, key, item):
-        super().__setitem__(key, item)
-        try:
-            exec(f'self.{key} = self["{key}"]')
-        except:
-            pass
-
-    def __delitem__(self, key):
-        try:
-            exec(f'del self.{key}')
-        except:
-            pass
-        super().__delitem__(key)
-
     def copy(self):
         new = type(self)()
         for key in self:
@@ -114,13 +102,23 @@ class InpSectionGeneric(CustomDictWithAttributes):
 
     :term:`dict-like <mapping>`"
     """
-    _section_label = ''
+    _label = ''
     """str: label of the section"""
 
     def __init__(self, *args, **kwargs):
         CustomDictWithAttributes.__init__(self, *args, **kwargs)
         self._inp = None
 
+    def __setitem__(self, key, item):
+        super().__setitem__(key, item)
+        if isinstance(key, str) and ' ' not in key:
+            super().__setattr__(key, item)
+
+    def __delitem__(self, key):
+        if hasattr(self, key):
+            super().__delattr__(key)
+        super().__delitem__(key)
+
     def set_parent_inp(self, inp):
         self._inp = inp
 
@@ -194,14 +192,26 @@ class InpSection(CustomDict):
     def get_parent_inp(self):
         return self._inp
 
-
     # def __repr__(self):
     #     # return CustomDict.__repr__(self)
     #     return str(self)
-    #
+
     # def __str__(self):
     #     return f'[{self._section_object._section_label}] '  # + ' | '.join((str(s) for s in self.keys()))
 
+    def __setitem__(self, key, value):
+        if isinstance(self._identifier, str):
+            if not isinstance(key, str):
+                raise UserWarning('Wrong key type')
+        else:
+            if len(self._identifier) != len(key):
+                raise UserWarning('Wrong number of keys')
+
+        if not isinstance(value, self._section_object):
+            raise UserWarning('Wrong section-object type')
+
+        super().__setitem__(key, value)
+
     @property
     def objects(self):
         """
@@ -221,6 +231,16 @@ class InpSection(CustomDict):
         """
         return self._section_object._identifier
 
+    @property
+    def _label(self):
+        """
+        get the label of the section
+
+        Returns:
+            str: label of the section
+        """
+        return self._section_object._section_label
+
     @property
     def _table_inp_export(self):
         # if the section can be exported (converted to a string) as a table.
@@ -272,12 +292,15 @@ class InpSection(CustomDict):
         """
         return section_class.create_section(lines)
 
-    @property
-    def _sorted_values(self):
-        for k in sorted(self.keys()):
+    def get_objects(self, sort_alphabetical=False):
+        keys = self.keys()
+        if sort_alphabetical:
+            keys = sorted(keys, key=natural_keys)
+
+        for k in keys:
             yield self[k]
 
-    def to_inp_lines(self, fast=False):
+    def to_inp_lines(self, fast=False, sort_objects_alphabetical=False):
         """
         convert the section to a multi-line ``.inp``-file conform string
 
@@ -289,6 +312,9 @@ class InpSection(CustomDict):
                 - :obj:`True`: if no special formation of the input file is needed
                 - :obj:`False`: section is converted into a table to prettify string output (slower)
 
+            sort_objects_alphabetical (bool): if objects in a section should be sorted alphabetical |
+                default: use order of the read inp-file and append new objects
+
         Returns:
              str: lines of the ``.inp``-file section
         """
@@ -296,10 +322,22 @@ class InpSection(CustomDict):
             return ';; No Data'
 
         if fast or not self._table_inp_export:
-            return '\n'.join(o.to_inp_line() for o in tqdm(self._sorted_values, desc=str(self._section_object.__name__),
-                                                           postfix='Write', total=len(self.keys())))
+
+            # only show write progress for big files
+            n_objects = len(self.keys())
+            values = self.get_objects(sort_objects_alphabetical)
+            if n_objects > 50_000:
+                _iterable = tqdm(values,
+                                 desc=self._section_object.__name__,
+                                 postfix='Write',
+                                 total=n_objects)
+            else:
+                _iterable = values
+
+            return '\n'.join(o.to_inp_line() for o in _iterable)
         else:
-            return dataframe_to_inp_string(self.frame)
+            return dataframe_to_inp_string(self.get_dataframe(set_index=True,
+                                                              sort_objects_alphabetical=sort_objects_alphabetical))
 
     @property
     def frame(self):
@@ -310,19 +348,22 @@ class InpSection(CustomDict):
         Returns:
             pandas.DataFrame: section as table
         """
-        return self.get_dataframe(set_index=True)
+        return self.get_dataframe(set_index=True, sort_objects_alphabetical=True)
 
-    def get_dataframe(self, set_index=True):
+    def get_dataframe(self, set_index=True, sort_objects_alphabetical=False):
         """convert section to a pandas data-frame
 
-       This property is used for debugging purposes and data analysis of the input data of the swmm model.
+        Args:
+            set_index (bool): set object keys as index
+            sort_objects_alphabetical (bool): if objects in a section should be sorted alphabetical |
+                default: use order of the read inp-file and append new objects
 
-       Returns:
-           pandas.DataFrame: section as table
-       """
+        Returns:
+            pandas.DataFrame: section as table
+        """
         if not self:  # if empty
             return DataFrame()
-        df = DataFrame([i.to_dict_() for i in self._sorted_values])
+        df = DataFrame([i.to_dict_() for i in self.get_objects(sort_objects_alphabetical)])
         if set_index:
             df = df.set_index(self._identifier)
         return df
@@ -462,6 +503,10 @@ class BaseSectionObject(ABC):
         # TODO: testing!!!
         return isinstance(self, type(other)) and all([is_equal(self[k], other[k]) for k in self.to_dict_().keys()])
 
+    def __hash__(self):
+        di = self.to_dict_()
+        return tuple([(k, v) for k, v in di.items()]).__hash__()
+
     # @property
     # def id(self):
     #     return id(self)
@@ -541,44 +586,24 @@ class BaseSectionObject(ABC):
             InpSection: new section of this object
         """
         sec = cls._section_class(cls)
+        # import sys
         if lines is not None:
-
-            def txt_to_lines(content):
-                """
-                converts text to multiple line arguments
-
-                Args:
-                    content (str): section text
-
-                Returns:
-                    list[list[str]]: lines in the input file section
-                """
-                for line in re.findall(r'^[ \t]*([^;\n]+)[ \t]*;?[^\n]*$', content, flags=re.M):
-                    # ;; section comment
-                    # ; object comment / either inline(at the end of the line) or before the line
-                    # if ';' in line:
-                    #     line
-                    # line = line.split(';')[0]
-                    # line = line.strip()
-                    # if line == '':  # ignore empty and comment lines
-                    #     continue
-                    # else:
-                    yield line.split()
-
+            # print(f'BYTES: {len(lines):>9_d}', )
             if isinstance(lines, str):
-                if len(lines) > 100000:
-                    n_lines = lines.count('\n') - 1
+                if len(lines) > 10_000_000:
+                    n_lines = lines.strip().count('\n') + 1
                     # to create a progressbar in the reading process
                     # only needed with big (> 200 MB) files
-                    lines = txt_to_lines(lines)
-                    lines = tqdm(lines, desc=cls.__name__, total=n_lines, postfix='Read')
+                    lines_iter = tqdm(txt_to_lines(lines), desc=cls.__name__, total=n_lines, postfix='Read')
                 else:
-                    lines = txt_to_lines(lines)
+                    lines_iter = txt_to_lines(lines)
+            else:
+                lines_iter = lines
 
-            sec.add_inp_lines(lines)
+            sec.add_inp_lines(lines_iter)
 
-            if isinstance(lines, tqdm):
-                lines.close()
+            if isinstance(lines_iter, tqdm):
+                lines_iter.close()
 
         return sec
 
@@ -620,7 +645,7 @@ def dataframe_to_inp_string(df):
     if df.empty:
         return ';; NO data'
 
-    c = df.sort_index().copy()
+    c = df.copy()
     if c.columns.name is None:
         c.columns.name = comment_sign
     else:
@@ -670,49 +695,118 @@ def convert_section(head, lines, converter):
         else:
             raise NotImplemented()
     else:
-        return lines.replace(inp_sep, '').strip()
+        return lines.replace(SEP_INP, '').strip()
 
 
 ########################################################################################################################
-def _sort_by(key):
-    sections_order = ([
-                          TITLE,
-                          OPTIONS,
-                          REPORT,
-                          EVAPORATION,
-                          TEMPERATURE
+SECTIONS_ORDER_MP = ([
+                         TITLE,
+                         OPTIONS,
+                         REPORT,
+                         EVAPORATION,
+                         TEMPERATURE
 
-                      ] + NODE_SECTIONS +
-                      [
-                          DWF,
-                          INFLOWS
+                     ] + NODE_SECTIONS +
+                     [
+                         DWF,
+                         INFLOWS
 
-                      ] + LINK_SECTIONS +
-                      [
+                     ] + LINK_SECTIONS +
+                     [
 
-                          LOSSES,
-                          XSECTIONS,
-                          TRANSECTS,
+                         LOSSES,
+                         XSECTIONS,
+                         TRANSECTS,
 
-                          CURVES,
-                          TIMESERIES,
-                          RAINGAGES,
-                          PATTERNS,
+                         CURVES,
+                         TIMESERIES,
+                         RAINGAGES,
+                         PATTERNS,
 
-                          SUBCATCHMENTS,
-                          SUBAREAS,
-                          INFILTRATION,
+                         SUBCATCHMENTS,
+                         SUBAREAS,
+                         INFILTRATION,
 
-                          POLLUTANTS,
-                          LOADINGS,
-                      ])
+                         POLLUTANTS,
+                         LOADINGS,
+                     ])
+
+SECTION_ORDER_DEFAULT = [TITLE,
+                         OPTIONS,
+                         FILES,
+                         EVAPORATION,
+                         TEMPERATURE,
+                         RAINGAGES,
+                         SUBCATCHMENTS,
+                         SUBAREAS,
+                         INFILTRATION,
+                         LID_CONTROLS,
+                         LID_USAGE,
+                         AQUIFERS,
+                         GROUNDWATER,
+                         GWF,
+                         SNOWPACKS,
+                         JUNCTIONS,
+                         OUTFALLS,
+                         STORAGE,
+                         CONDUITS,
+                         PUMPS,
+                         ORIFICES,
+                         WEIRS,
+                         OUTLETS,
+                         XSECTIONS,
+                         TRANSECTS,
+                         LOSSES,
+                         CONTROLS,
+                         POLLUTANTS,
+                         LANDUSES,
+                         COVERAGES,
+                         LOADINGS,
+                         BUILDUP,
+                         WASHOFF,
+                         TREATMENT,
+                         INFLOWS,
+                         DWF,
+                         HYDROGRAPHS,
+                         RDII,
+                         CURVES,
+                         TIMESERIES,
+                         PATTERNS,
+                         REPORT,
+                         ADJUSTMENTS,
+                         TAGS,
+                         MAP,
+                         COORDINATES,
+                         VERTICES,
+                         POLYGONS,
+                         SYMBOLS,
+                         LABELS,
+                         BACKDROP,
+                         PROFILES]
+
+
+def check_order(inp, order_list=None):
+    if order_list is None:
+        order_list = SECTION_ORDER_DEFAULT
+    order = [order_list.index(o) if o in order_list else len(order_list) for o in inp]
+    return all((order[i+1] - order[i]) > 0 for i in range(len(order)-1))
+
+
+def _sort_by(key, sections_order):
     if key in sections_order:
         return sections_order.index(key)
     else:
         return len(sections_order)
 
 
-def section_to_string(section, fast=True):
+re_int = re.compile('(\d+)')
+
+
+def natural_keys(text):
+    return [int(text) if text.isdigit() else text for text in re_int.split(text)]
+
+
+def section_to_string(section, fast=True, sort_objects_alphabetical=False):
     """
     create a string of a section in an ``.inp``-file
 
@@ -720,6 +814,8 @@ def section_to_string(section, fast=True):
         section (InpSection | InpSectionGeneric):
             section of an ``.inp``-file
         fast (bool): don't use any formatting else format as table
+        sort_objects_alphabetical (bool): if objects in a section should be sorted alphabetical |
+            default: use order of the read inp-file and append new objects
 
     Returns:
         str: string of the ``.inp``-file section
@@ -728,34 +824,34 @@ def section_to_string(section, fast=True):
 
     # ----------------------
     if isinstance(section, str):  # Title
-        f += section.replace(inp_sep, '').strip()
+        f += section.replace(SEP_INP, '').strip()
 
     # ----------------------
-    # elif isinstance(section, list):  # V0.1
-    #     for line in section:
-    #         f += type2str(line) + '\n'
+    elif isinstance(section, list):  # V0.1
+        for line in section:
+            f += type2str(line) + '\n'
     #
     # # ----------------------
-    # elif isinstance(section, dict):  # V0.2
-    #     max_len = len(max(section.keys(), key=len)) + 2
-    #     for sub in section:
-    #         f += '{key}{value}'.format(key=sub.ljust(max_len),
-    #                                    value=type2str(section[sub]) + '\n')
+    elif isinstance(section, dict):  # V0.2
+        max_len = len(max(section.keys(), key=len)) + 2
+        for sub in section:
+            f += '{key}{value}'.format(key=sub.ljust(max_len),
+                                       value=type2str(section[sub]) + '\n')
     #
     # ----------------------
-    # elif isinstance(section, (DataFrame, Series)):  # V0.3
-    #     if section.empty:
-    #         f += ';; NO data'
-    #
-    #     if isinstance(section, DataFrame):
-    #         f += dataframe_to_inp_string(section)
-    #
-    #     elif isinstance(section, Series):
-    #         f += section.apply(type2str).to_string()
+    elif isinstance(section, (DataFrame, Series)):  # V0.3
+        if section.empty:
+            f += ';; NO data'
+
+        if isinstance(section, DataFrame):
+            f += dataframe_to_inp_string(section)
+
+        elif isinstance(section, Series):
+            f += section.apply(type2str).to_string()
 
     # ----------------------
     elif isinstance(section, (InpSection, InpSectionGeneric)):  # V0.4
-        f += section.to_inp_lines(fast=fast)
+        f += section.to_inp_lines(fast=fast, sort_objects_alphabetical=sort_objects_alphabetical)
 
     # ----------------------
     f += '\n'
