@@ -1,37 +1,34 @@
-from numpy import interp, mean, ceil
 from networkx import DiGraph
+from numpy import interp, mean, ceil
 
-from .collection import nodes_dict, links_dict, subcachtment_nodes_dict
+from .collection import nodes_dict, links_dict, subcatchments_per_node_dict
 from .graph import next_links_labels, previous_links, previous_links_labels, links_connected
-from .macros import calc_slope, find_link
-from ..section_abr import SEC
-from ..section_lists import NODE_SECTIONS, LINK_SECTIONS, SUBCATCHMENT_SECTIONS
+from .macros import calc_slope, find_link, delete_sections
+from ..inp import SwmmInput
+from ..section_labels import *
+from ..section_lists import NODE_SECTIONS, LINK_SECTIONS, SUBCATCHMENT_SECTIONS, POLLUTANT_SECTIONS
 from ..sections import Tag, DryWeatherFlow, Junction, Coordinate, Conduit, Loss, Vertices, EvaporationSection
-from ... import SwmmInput
 
 
 def delete_node(inp: SwmmInput, node_label, graph: DiGraph = None, alt_node=None):
     """
     delete node in inp data
 
-    Notes:
-        works inplace
-
     Args:
         inp (SwmmInput): inp data
         node_label (str): label of node to delete
         graph (DiGraph): networkx graph of model
         alt_node (str): node label | optional: move flows to this node
 
-    Returns:
-        SwmmInput: inp data
+    .. Important::
+        works inplace
     """
-    for section in NODE_SECTIONS + [SEC.COORDINATES]:
+    for section in NODE_SECTIONS + [COORDINATES]:
         if (section in inp) and (node_label in inp[section]):
             inp[section].pop(node_label)
 
-    if (SEC.TAGS in inp) and ((Tag.TYPES.Node, node_label) in inp.TAGS):
-        del inp[SEC.TAGS][(Tag.TYPES.Node, node_label)]
+    if (TAGS in inp) and ((Tag.TYPES.Node, node_label) in inp.TAGS):
+        del inp[TAGS][(Tag.TYPES.Node, node_label)]
 
     # AND delete connected links
     if graph is not None:
@@ -54,8 +51,6 @@ def delete_node(inp: SwmmInput, node_label, graph: DiGraph = None, alt_node=None
     if alt_node is not None:
         move_flows(inp, node_label, alt_node)
 
-    return inp
-
 
 def move_flows(inp: SwmmInput, from_node, to_node, only_constituent=None):
     """
@@ -70,7 +65,7 @@ def move_flows(inp: SwmmInput, from_node, to_node, only_constituent=None):
     Notes:
         works inplace
     """
-    for section in (SEC.INFLOWS, SEC.DWF):
+    for section in (INFLOWS, DWF):
         if section not in inp:
             continue
 
@@ -89,7 +84,7 @@ def move_flows(inp: SwmmInput, from_node, to_node, only_constituent=None):
                 if index_new not in inp[section]:
                     inp[section].add_obj(obj)
 
-                elif section == SEC.DWF:
+                elif section == DWF:
                     # DryWeatherFlow can be easily added when Patterns are equal
                     inp[section][index_new].Base += obj.Base
 
@@ -97,7 +92,7 @@ def move_flows(inp: SwmmInput, from_node, to_node, only_constituent=None):
                     #     print(f'WARNING: move_flows  from "{from_node}" to "{to_node}". DWF patterns don\'t
                     #     match!')
 
-                elif section == SEC.INFLOWS:
+                elif section == INFLOWS:
                     # Inflows can't be added due to the multiplication factor / timeseries
                     # if (TimeSeries, Type, Mfactor, Sfactor,Pattern) are equal then sum(Baseline)
                     print(f'WARNING: move_flows  from "{from_node}" to "{to_node}". Already Exists!')
@@ -107,29 +102,29 @@ def move_flows(inp: SwmmInput, from_node, to_node, only_constituent=None):
 
 
 def delete_link(inp: SwmmInput, link):
-    for s in LINK_SECTIONS + [SEC.XSECTIONS, SEC.LOSSES, SEC.VERTICES]:
+    for s in LINK_SECTIONS + [XSECTIONS, LOSSES, VERTICES]:
         if (s in inp) and (link in inp[s]):
             inp[s].pop(link)
 
-    if (SEC.TAGS in inp) and ((Tag.TYPES.Link, link) in inp.TAGS):
-        del inp[SEC.TAGS][(Tag.TYPES.Link, link)]
+    if (TAGS in inp) and ((Tag.TYPES.Link, link) in inp.TAGS):
+        del inp[TAGS][(Tag.TYPES.Link, link)]
 
 
 def delete_subcatchment(inp: SwmmInput, subcatchment):
-    for s in SUBCATCHMENT_SECTIONS + [SEC.LOADINGS, SEC.COVERAGES]:
-        # , SEC.GWF, SEC.GROUNDWATER
+    for s in SUBCATCHMENT_SECTIONS + [LOADINGS, COVERAGES]:
+        # , GWF, GROUNDWATER
         if (s in inp) and (subcatchment in inp[s]):
             inp[s].pop(subcatchment)
 
-    if (SEC.TAGS in inp) and ((Tag.TYPES.Subcatch, subcatchment) in inp.TAGS):
-        del inp[SEC.TAGS][(Tag.TYPES.Subcatch, subcatchment)]
+    if (TAGS in inp) and ((Tag.TYPES.Subcatch, subcatchment) in inp.TAGS):
+        del inp[TAGS][(Tag.TYPES.Subcatch, subcatchment)]
 
 
 def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
     # mode = [cut_point (GUI), intervals (n), length (l)]
     nodes = nodes_dict(inp)
     if isinstance(conduit, str):
-        conduit = inp[SEC.CONDUITS][conduit]  # type: Conduit
+        conduit = inp[CONDUITS][conduit]  # type: Conduit
 
     dx = 0
     n_new_nodes = 0
@@ -143,12 +138,12 @@ def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
     from_node = nodes[conduit.FromNode]
     to_node = nodes[conduit.ToNode]
 
-    from_node_coord = inp[SEC.COORDINATES][from_node.Name]
-    to_node_coord = inp[SEC.COORDINATES][to_node.Name]
+    from_node_coord = inp[COORDINATES][from_node.Name]
+    to_node_coord = inp[COORDINATES][to_node.Name]
 
     loss = None
-    if (SEC.LOSSES in inp) and (conduit.Name in inp[SEC.LOSSES]):
-        loss = inp[SEC.LOSSES][conduit.Name]  # type: Loss
+    if (LOSSES in inp) and (conduit.Name in inp[LOSSES]):
+        loss = inp[LOSSES][conduit.Name]  # type: Loss
 
     new_nodes = list()
     new_links = list()
@@ -167,10 +162,10 @@ def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
                             Aponded=float(mean([from_node.Aponded, to_node.Aponded])),
                             )
             new_nodes.append(node)
-            inp[SEC.JUNCTIONS].add_obj(node)
+            inp[JUNCTIONS].add_obj(node)
 
             # TODO: COORDINATES based on vertices
-            inp[SEC.COORDINATES].add_obj(Coordinate(node.Name,
+            inp[COORDINATES].add_obj(Coordinate(node.Name,
                                                     x=interp(x, [0, conduit.Length],
                                                                 [from_node_coord.x, to_node_coord.x]),
                                                     y=interp(x, [0, conduit.Length],
@@ -186,11 +181,11 @@ def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
                        InitFlow=conduit.InitFlow,
                        MaxFlow=conduit.MaxFlow)
         new_links.append(link)
-        inp[SEC.CONDUITS].add_obj(link)
+        inp[CONDUITS].add_obj(link)
 
-        xs = inp[SEC.XSECTIONS][conduit.Name].copy()
+        xs = inp[XSECTIONS][conduit.Name].copy()
         xs.Link = link.Name
-        inp[SEC.XSECTIONS].add_obj(xs)
+        inp[XSECTIONS].add_obj(xs)
 
         if loss:
             inlet = loss.Inlet if loss.Inlet and (new_node_i == 0) else 0
@@ -199,7 +194,7 @@ def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
             flap_gate = loss.FlapGate
 
             if any([inlet, outlet, average, flap_gate]):
-                inp[SEC.LOSSES].add_obj(Loss(link.Name, inlet, outlet, average, flap_gate))
+                inp[LOSSES].add_obj(Loss(link.Name, inlet, outlet, average, flap_gate))
 
         # TODO: VERTICES
 
@@ -208,7 +203,7 @@ def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
         last_node = node
         x += dx
 
-    # if conduit.Name in inp[SEC.VERTICES]:
+    # if conduit.Name in inp[VERTICES]:
     #     pass
     # else:
     #     # interpolate coordinates
@@ -218,42 +213,40 @@ def split_conduit(inp, conduit, intervals=None, length=None, from_inlet=True):
 
 
 def combine_vertices(inp: SwmmInput, label1, label2):
-    if SEC.COORDINATES not in inp:
+    if COORDINATES not in inp:
         # if there are not coordinates this function is nonsense
         return
 
     vertices_class = Vertices
 
-    if SEC.VERTICES not in inp:
+    if VERTICES not in inp:
         # we will at least ad the coordinates of the common node
-        inp[SEC.VERTICES] = Vertices.create_section()
+        inp[VERTICES] = Vertices.create_section()
     else:
-        vertices_class = inp[SEC.VERTICES]._section_object
+        vertices_class = inp[VERTICES]._section_object
 
     new_vertices = list()
 
-    if label1 in inp[SEC.VERTICES]:
-        new_vertices += list(inp[SEC.VERTICES][label1].vertices)
+    if label1 in inp[VERTICES]:
+        new_vertices += list(inp[VERTICES][label1].vertices)
 
     common_node = links_dict(inp)[label1].ToNode
-    if common_node in inp[SEC.COORDINATES]:
-        new_vertices += [inp[SEC.COORDINATES][common_node].point]
+    if common_node in inp[COORDINATES]:
+        new_vertices += [inp[COORDINATES][common_node].point]
 
-    if label2 in inp[SEC.VERTICES]:
-        new_vertices += list(inp[SEC.VERTICES][label2].vertices)
+    if label2 in inp[VERTICES]:
+        new_vertices += list(inp[VERTICES][label2].vertices)
 
-    if label1 in inp[SEC.VERTICES]:
-        inp[SEC.VERTICES][label1].vertices = new_vertices
+    if label1 in inp[VERTICES]:
+        inp[VERTICES][label1].vertices = new_vertices
     else:
-        inp[SEC.VERTICES].add_obj(vertices_class(label1, vertices=new_vertices))
+        inp[VERTICES].add_obj(vertices_class(label1, vertices=new_vertices))
 
 
 def combine_conduits(inp, c1, c2, graph: DiGraph = None):
     """
     combine the two conduits to one keep attributes of the first (c1)
 
-    works inplace
-
     Args:
         inp (SwmmInput): inp data
         c1 (str | Conduit): conduit 1 to combine
@@ -261,12 +254,15 @@ def combine_conduits(inp, c1, c2, graph: DiGraph = None):
         graph (networkx.DiGraph): optional, runs faster with graph (inp representation)
 
     Returns:
-        SwmmInput: inp data
+        Conduit: new combined conduit
+
+    .. Important::
+        works inplace
     """
     if isinstance(c1, str):
-        c1 = inp[SEC.CONDUITS][c1]
+        c1 = inp[CONDUITS][c1]
     if isinstance(c2, str):
-        c2 = inp[SEC.CONDUITS][c2]
+        c2 = inp[CONDUITS][c2]
     # -------------------------
     if graph:
         graph.remove_edge(c1.FromNode, c1.ToNode)
@@ -302,7 +298,7 @@ def combine_conduits(inp, c1, c2, graph: DiGraph = None):
         c_new.OutOffset = c_second.OutOffset
 
     # Loss
-    if (SEC.LOSSES in inp) and (c_new.Name in inp[SEC.LOSSES]):
+    if (LOSSES in inp) and (c_new.Name in inp[LOSSES]):
         print(f'combine_conduits {c1.Name} and {c2.Name}. BUT WHAT TO DO WITH LOSSES?')
         # add losses
         pass
@@ -351,7 +347,7 @@ def dissolve_conduit(inp, c: Conduit, graph: DiGraph = None):
             graph.add_edge(c_new.FromNode, c_new.ToNode, label=c_new.Name)
 
         # Loss
-        if SEC.LOSSES in inp and c_new.Name in inp[SEC.LOSSES]:
+        if LOSSES in inp and c_new.Name in inp[LOSSES]:
             print(f'dissolve_conduit {c.Name} in {c_new.Name}. BUT WHAT TO DO WITH LOSSES?')
 
         if isinstance(c_new, Conduit):
@@ -366,20 +362,20 @@ def rename_node(inp: SwmmInput, old_label: str, new_label: str, g=None):
     """
     change node label
 
-    Notes:
-        works inplace
-        CONTROLS Not Implemented!
-
     Args:
         inp (SwmmInput): inp data
         old_label (str): previous node label
         new_label (str): new node label
-        g (
+        g (DiGraph): optional - graph of the network
+
+    .. Important::
+        works inplace
+        CONTROLS Not Implemented!
     """
     # ToDo: Not Implemented: CONTROLS
 
     # Nodes and basic node components
-    for section in NODE_SECTIONS + [SEC.COORDINATES, SEC.RDII]:
+    for section in NODE_SECTIONS + [COORDINATES, RDII]:
         if (section in inp) and (old_label in inp[section]):
             inp[section][new_label] = inp[section].pop(old_label)
             if hasattr(inp[section][new_label], 'Name'):
@@ -388,14 +384,14 @@ def rename_node(inp: SwmmInput, old_label: str, new_label: str, g=None):
                 inp[section][new_label].Node = new_label
 
     # tags
-    if (SEC.TAGS in inp) and ((Tag.TYPES.Node, old_label) in inp.TAGS):
-        tag = inp[SEC.TAGS].pop((Tag.TYPES.Node, old_label))
+    if (TAGS in inp) and ((Tag.TYPES.Node, old_label) in inp.TAGS):
+        tag = inp[TAGS].pop((Tag.TYPES.Node, old_label))
         tag.Name = new_label
         inp.TAGS.add_obj(tag)
 
     # subcatchment outlets
-    if SEC.SUBCATCHMENTS in inp:
-        for obj in subcachtment_nodes_dict(inp)[old_label]:
+    if SUBCATCHMENTS in inp:
+        for obj in subcatchments_per_node_dict(inp)[old_label]:
             obj.Outlet = new_label
         # -------
         # for obj in inp.SUBCATCHMENTS.filter_keys([old_label], 'Outlet'):  # type: SubCatchment
@@ -411,7 +407,7 @@ def rename_node(inp: SwmmInput, old_label: str, new_label: str, g=None):
         link.ToNode = new_label
 
     # -------
-    # for section in [SEC.CONDUITS, SEC.PUMPS, SEC.ORIFICES, SEC.WEIRS, SEC.OUTLETS]:
+    # for section in [CONDUITS, PUMPS, ORIFICES, WEIRS, OUTLETS]:
     #     if section in inp:
     #         for obj in inp[section].filter_keys([old_label], 'FromNode'):  # type: _Link
     #             obj.FromNode = new_label
@@ -422,10 +418,10 @@ def rename_node(inp: SwmmInput, old_label: str, new_label: str, g=None):
 
     # (dwf-)inflows
     constituents = [DryWeatherFlow.TYPES.FLOW]
-    if SEC.POLLUTANTS in inp:
+    if POLLUTANTS in inp:
         constituents += list(inp.POLLUTANTS.keys())
 
-    for section in [SEC.INFLOWS, SEC.DWF, SEC.TREATMENT]:
+    for section in [INFLOWS, DWF, TREATMENT]:
         if section in inp:
             for constituent in constituents:
                 old_id = (old_label, constituent)
@@ -455,7 +451,7 @@ def rename_link(inp: SwmmInput, old_label: str, new_label: str):
         new_label (str): new link label
     """
     # ToDo: Not Implemented: CONTROLS
-    for section in LINK_SECTIONS + [SEC.XSECTIONS, SEC.LOSSES, SEC.VERTICES]:
+    for section in LINK_SECTIONS + [XSECTIONS, LOSSES, VERTICES]:
         if (section in inp) and (old_label in inp[section]):
             inp[section][new_label] = inp[section].pop(old_label)
             if hasattr(inp[section][new_label], 'Name'):
@@ -463,12 +459,12 @@ def rename_link(inp: SwmmInput, old_label: str, new_label: str):
             else:
                 inp[section][new_label].Link = new_label
 
-    if (SEC.TAGS in inp) and ((Tag.TYPES.Link, old_label) in inp.TAGS):
-        inp[SEC.TAGS][(Tag.TYPES.Link, new_label)] = inp[SEC.TAGS].pop((Tag.TYPES.Link, old_label))
+    if (TAGS in inp) and ((Tag.TYPES.Link, old_label) in inp.TAGS):
+        inp[TAGS][(Tag.TYPES.Link, new_label)] = inp[TAGS].pop((Tag.TYPES.Link, old_label))
 
 
 def rename_subcatchment(inp: SwmmInput, old_label: str, new_label: str):
-    for section in SUBCATCHMENT_SECTIONS + [SEC.LOADINGS, SEC.COVERAGES, SEC.GWF, SEC.GROUNDWATER]:
+    for section in SUBCATCHMENT_SECTIONS + [LOADINGS, COVERAGES, GWF, GROUNDWATER]:
         if (section in inp) and (old_label in inp[section]):
             inp[section][new_label] = inp[section].pop(old_label)
             if hasattr(inp[section][new_label], 'Name'):
@@ -476,61 +472,79 @@ def rename_subcatchment(inp: SwmmInput, old_label: str, new_label: str):
             else:
                 inp[section][new_label].Subcatch = new_label
 
-    if (SEC.TAGS in inp) and ((Tag.TYPES.Subcatch, old_label) in inp.TAGS):
-        inp[SEC.TAGS][(Tag.TYPES.Subcatch, new_label)] = inp[SEC.TAGS].pop((Tag.TYPES.Subcatch, old_label))
+    if (TAGS in inp) and ((Tag.TYPES.Subcatch, old_label) in inp.TAGS):
+        inp[TAGS][(Tag.TYPES.Subcatch, new_label)] = inp[TAGS].pop((Tag.TYPES.Subcatch, old_label))
 
 
 def rename_timeseries(inp, old_label, new_label):
     """
     change timeseries label
 
-    Notes:
-        works inplace
-
     Args:
         inp (SwmmInput): inp data
         old_label (str): previous timeseries label
         new_label (str): new timeseries label
+
+    .. Important::
+        works inplace
     """
-    if old_label in inp[SEC.TIMESERIES]:
-        obj = inp[SEC.TIMESERIES].pop(old_label)
+    if old_label in inp[TIMESERIES]:
+        obj = inp[TIMESERIES].pop(old_label)
         obj.Name = new_label
-        inp[SEC.TIMESERIES].add_obj(obj)
+        inp[TIMESERIES].add_obj(obj)
 
     key = EvaporationSection.KEYS.TIMESERIES  # TemperatureSection.KEYS.TIMESERIES, ...
 
-    if SEC.RAINGAGES in inp:
-        f = inp[SEC.RAINGAGES].frame
+    if RAINGAGES in inp:
+        f = inp[RAINGAGES].frame
         filtered_table = f[(f['Source'] == key) & (f['Timeseries'] == old_label)]
         if not filtered_table.empty:
             for i in filtered_table.index:
-                inp[SEC.RAINGAGES][i].Timeseries = new_label
+                inp[RAINGAGES][i].Timeseries = new_label
 
-    if SEC.EVAPORATION in inp:
-        if key in inp[SEC.EVAPORATION]:
-            if inp[SEC.EVAPORATION][key] == old_label:
-                inp[SEC.EVAPORATION][key] = new_label
+    if EVAPORATION in inp:
+        if key in inp[EVAPORATION]:
+            if inp[EVAPORATION][key] == old_label:
+                inp[EVAPORATION][key] = new_label
 
-    if SEC.TEMPERATURE in inp:
-        if key in inp[SEC.TEMPERATURE]:
-            if inp[SEC.TEMPERATURE][key] == old_label:
-                inp[SEC.TEMPERATURE][key] = new_label
+    if TEMPERATURE in inp:
+        if key in inp[TEMPERATURE]:
+            if inp[TEMPERATURE][key] == old_label:
+                inp[TEMPERATURE][key] = new_label
 
-    if SEC.OUTFALLS in inp:
-        f = inp[SEC.OUTFALLS].frame
+    if OUTFALLS in inp:
+        f = inp[OUTFALLS].frame
         filtered_table = f[(f['Type'] == key) & (f['Data'] == old_label)]
         if not filtered_table.empty:
             for i in filtered_table.index:
-                inp[SEC.OUTFALLS][i].Data = new_label
+                inp[OUTFALLS][i].Data = new_label
 
-    if SEC.INFLOWS in inp:
-        f = inp[SEC.INFLOWS].frame
+    if INFLOWS in inp:
+        f = inp[INFLOWS].frame
         filtered_table = f[f['TimeSeries'] == old_label]
         if not filtered_table.empty:
             for i in filtered_table.index:
-                inp[SEC.INFLOWS][i].TimeSeries = new_label
+                inp[INFLOWS][i].TimeSeries = new_label
 
 
 def flip_link_direction(inp, link_label):
     link = find_link(inp, link_label)
     link.FromNode, link.ToNode = link.ToNode, link.FromNode
+
+
+def remove_quality_model(inp):
+    """
+    remove all sections only for modelling quality
+
+    Args:
+        inp (SwmmInput): inp-data
+
+    .. Important::
+        works inplace
+    """
+    delete_sections(inp, POLLUTANT_SECTIONS)
+
+    for sec in [INFLOWS, DWF]:
+        for k in list(inp[sec].keys()):
+            if inp[sec][k].Constituent != 'FLOW':
+                del inp[sec][k]
